@@ -67,6 +67,9 @@ static struct timeval last_wakeup;
 /** current limit for the main loop's runtime */
 static float main_loop_iteration_limit = 0.1;
 
+/** A pipe that is used to asynchronously handle SIGCHLD */
+static int sigchld_pipe[2];
+
 /** Call before exiting.
  */
 void
@@ -115,6 +118,9 @@ awesome_atexit(bool restart)
     /* Disconnect *after* closing lua */
     xcb_cursor_context_free(globalconf.cursor_ctx);
     xcb_disconnect(globalconf.connection);
+
+    close(sigchld_pipe[0]);
+    close(sigchld_pipe[1]);
 }
 
 /** Restore the client order after a restart */
@@ -158,7 +164,6 @@ scan(xcb_query_tree_cookie_t tree_c)
     xcb_get_window_attributes_reply_t *attr_r;
     xcb_get_geometry_reply_t *geom_r;
     xcb_get_property_cookie_t prop_cookie;
-    long state;
 
     tree_r = xcb_query_tree_reply(globalconf.connection,
                                   tree_c,
@@ -197,7 +202,7 @@ scan(xcb_query_tree_cookie_t tree_c)
                                                  NULL);
         geom_r = xcb_get_geometry_reply(globalconf.connection, geom_wins[i], NULL);
 
-        state = xwindow_get_state_reply(state_wins[i]);
+        long state = xwindow_get_state_reply(state_wins[i]);
 
         if(!geom_r || !attr_r || attr_r->override_redirect
            || attr_r->map_state == XCB_MAP_STATE_UNMAPPED
@@ -405,6 +410,7 @@ a_glib_poll(GPollFD *ufds, guint nfsd, gint timeout)
     guint res;
     struct timeval now, length_time;
     float length;
+    int saved_errno;
     lua_State *L = globalconf_get_lua_State();
 
     /* Do all deferred work now */
@@ -435,8 +441,10 @@ a_glib_poll(GPollFD *ufds, guint nfsd, gint timeout)
 
     /* Actually do the polling, record time of wakeup and check for new xcb events */
     res = g_poll(ufds, nfsd, timeout);
+    saved_errno = errno;
     gettimeofday(&last_wakeup, NULL);
     a_xcb_check();
+    errno = saved_errno;
 
     return res;
 }
@@ -449,6 +457,35 @@ signal_fatal(int signum)
     fatal("signal %d, dumping backtrace\n%s", signum, buf.s);
 }
 
+/* Signal handler for SIGCHLD. Causes reap_children() to be called. */
+static void
+signal_child(int signum)
+{
+    assert(signum == SIGCHLD);
+    int res = write(sigchld_pipe[1], " ", 1);
+    (void) res;
+    assert(res == 1);
+}
+
+/* There was a SIGCHLD signal. Read from sigchld_pipe and reap children. */
+static gboolean
+reap_children(GIOChannel *channel, GIOCondition condition, gpointer user_data)
+{
+    pid_t child;
+    int status;
+    char buffer[1024];
+    ssize_t result = read(sigchld_pipe[0], &buffer[0], sizeof(buffer));
+    if (result < 0)
+        fatal("Error reading from signal pipe: %s", strerror(errno));
+
+    while ((child = waitpid(-1, &status, WNOHANG)) > 0) {
+        spawn_child_exited(child, status);
+    }
+    if (child < 0 && errno != ECHILD)
+        warn("waitpid(-1) failed: %s", strerror(errno));
+    return TRUE;
+}
+
 /** Function to exit on some signals.
  * \param data currently unused
  */
@@ -463,7 +500,7 @@ void
 awesome_restart(void)
 {
     awesome_atexit(true);
-    execvp(awesome_argv[0], spawn_transform_commandline(awesome_argv));
+    execvp(awesome_argv[0], awesome_argv);
     fatal("execv() failed: %s", strerror(errno));
 }
 
@@ -578,7 +615,7 @@ main(int argc, char **argv)
             replace_wm = true;
             break;
           case '\1':
-            spawn_handle_reap(optarg);
+            /* Silently ignore --reap and its argument */
             break;
           default:
             exit_help(EXIT_FAILURE);
@@ -586,7 +623,8 @@ main(int argc, char **argv)
         }
 
     /* Get XDG basedir data */
-    xdgInitHandle(&xdg);
+    if(!xdgInitHandle(&xdg))
+        fatal("Function xdgInitHandle() failed, is $HOME unset?");
 
     /* add XDG_CONFIG_DIR as include path */
     const char * const *xdgconfigdirs = xdgSearchableConfigDirectories(&xdg);
@@ -630,6 +668,16 @@ main(int argc, char **argv)
         }
     }
 
+    /* Setup pipe for SIGCHLD processing */
+    {
+        if (!g_unix_open_pipe(sigchld_pipe, FD_CLOEXEC, NULL))
+            fatal("Failed to create pipe");
+
+        GIOChannel *channel = g_io_channel_unix_new(sigchld_pipe[0]);
+        g_io_add_watch(channel, G_IO_IN, reap_children, NULL);
+        g_io_channel_unref(channel);
+    }
+
     /* register function for signals */
     g_unix_signal_add(SIGINT, exit_on_signal, NULL);
     g_unix_signal_add(SIGTERM, exit_on_signal, NULL);
@@ -644,6 +692,10 @@ main(int argc, char **argv)
     sigaction(SIGSEGV, &sa, 0);
     signal(SIGPIPE, SIG_IGN);
 
+    sa.sa_handler = signal_child;
+    sa.sa_flags = SA_NOCLDSTOP | SA_RESTART;
+    sigaction(SIGCHLD, &sa, 0);
+
     /* We have no clue where the input focus is right now */
     globalconf.focus.need_update = true;
 
