@@ -23,6 +23,7 @@
 #define AWESOME_COMMON_LUALIB
 
 #include <lua.h>
+#include <lauxlib.h>
 
 #include "common/util.h"
 
@@ -84,6 +85,40 @@ luaA_dofunction(lua_State *L, int nargs, int nret)
     return true;
 }
 
+/** Call a registered function. Its arguments are the complete stack contents.
+ * \param L The Lua VM state.
+ * \param handler The function to call.
+ * \return The number of elements pushed on stack.
+ */
+static inline
+int luaA_call_handler(lua_State *L, int handler)
+{
+    /* This is based on luaA_dofunction, but allows multiple return values */
+    assert(handler != LUA_REFNIL);
+
+    int nargs = lua_gettop(L);
+
+    /* Push error handling function and move it before args */
+    lua_pushcfunction(L, luaA_dofunction_error);
+    lua_insert(L, - nargs - 1);
+    int error_func_pos = 1;
+
+    /* push function and move it before args */
+    lua_rawgeti(L, LUA_REGISTRYINDEX, handler);
+    lua_insert(L, - nargs - 1);
+
+    if(lua_pcall(L, nargs, LUA_MULTRET, error_func_pos))
+    {
+        warn("%s", lua_tostring(L, -1));
+        /* Remove error function and error string */
+        lua_pop(L, 2);
+        return 0;
+    }
+    /* Remove error function */
+    lua_remove(L, error_func_pos);
+    return lua_gettop(L);
+}
+
 #endif
 
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
