@@ -23,5 +23,9 @@
 #define AWESOME_COMMON_VERSION_H
 
 void eprint_version(void) __attribute__ ((noreturn));
+const char *awesome_version_string(void);
+const char *awesome_release_string(void);
 
 #endif
+
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
