@@ -329,6 +329,14 @@ event_handle_configurerequest(xcb_configure_request_event_t *ev)
 
     if((c = client_getbywin(ev->window)))
     {
+        lua_State *L = globalconf_get_lua_State();
+        if(ev->value_mask & XCB_CONFIG_WINDOW_BORDER_WIDTH)
+        {
+            luaA_object_push(L, c);
+            window_set_border_width(L, -1, ev->border_width);
+            lua_pop(L, 1);
+        }
+
         area_t geometry = c->geometry;
         uint16_t bw = c->border_width;
         uint16_t tb_left = c->titlebar[CLIENT_TITLEBAR_LEFT].size;
@@ -339,24 +347,8 @@ event_handle_configurerequest(xcb_configure_request_event_t *ev)
         uint16_t deco_right = bw + tb_right;
         uint16_t deco_top = bw + tb_top;
         uint16_t deco_bottom = bw + tb_bottom;
-        int16_t diff_w = 0, diff_h = 0, diff_border = 0;
+        int16_t diff_w = 0, diff_h = 0;
 
-        lua_State *L = globalconf_get_lua_State();
-
-        if(ev->value_mask & XCB_CONFIG_WINDOW_X)
-        {
-            int16_t diff = 0;
-            geometry.x = ev->x;
-            xwindow_translate_for_gravity(c->size_hints.win_gravity, deco_left, 0, deco_right, 0, &diff, NULL);
-            geometry.x += diff;
-        }
-        if(ev->value_mask & XCB_CONFIG_WINDOW_Y)
-        {
-            int16_t diff = 0;
-            geometry.y = ev->y;
-            xwindow_translate_for_gravity(c->size_hints.win_gravity, 0, deco_top, 0, deco_bottom, NULL, &diff);
-            geometry.y += diff;
-        }
         if(ev->value_mask & XCB_CONFIG_WINDOW_WIDTH)
         {
             uint16_t old_w = geometry.width;
@@ -373,26 +365,23 @@ event_handle_configurerequest(xcb_configure_request_event_t *ev)
             geometry.height += tb_top + tb_bottom;
             diff_h = geometry.height - old_h;
         }
-        if(ev->value_mask & XCB_CONFIG_WINDOW_BORDER_WIDTH)
-        {
-            diff_border = ev->border_width - bw;
-            diff_h += diff_border;
-            diff_w += diff_border;
-
-            luaA_object_push(L, c);
-            window_set_border_width(L, -1, ev->border_width);
-            lua_pop(L, 1);
-        }
 
         /* If the client resizes without moving itself, apply window gravity */
         if(c->size_hints.flags & XCB_ICCCM_SIZE_HINT_P_WIN_GRAVITY)
         {
-            int16_t diff_x = 0, diff_y = 0;
-            xwindow_translate_for_gravity(c->size_hints.win_gravity, diff_border, diff_border, diff_w, diff_h, &diff_x, &diff_y);
-            if(!(ev->value_mask & XCB_CONFIG_WINDOW_X))
-                geometry.x += diff_x;
-            if(!(ev->value_mask & XCB_CONFIG_WINDOW_Y))
-                geometry.y += diff_y;
+            xwindow_translate_for_gravity(c->size_hints.win_gravity, 0, 0, diff_w, diff_h, &geometry.x, &geometry.y);
+        }
+        if(ev->value_mask & XCB_CONFIG_WINDOW_X)
+        {
+            geometry.x = ev->x;
+            if(c->size_hints.flags & XCB_ICCCM_SIZE_HINT_P_WIN_GRAVITY)
+                xwindow_translate_for_gravity(c->size_hints.win_gravity, deco_left, 0, deco_right, 0, &geometry.x, NULL);
+        }
+        if(ev->value_mask & XCB_CONFIG_WINDOW_Y)
+        {
+            geometry.y = ev->y;
+            if(c->size_hints.flags & XCB_ICCCM_SIZE_HINT_P_WIN_GRAVITY)
+                xwindow_translate_for_gravity(c->size_hints.win_gravity, 0, deco_top, 0, deco_bottom, NULL, &geometry.y);
         }
 
         c->got_configure_request = true;
@@ -593,22 +582,42 @@ event_handle_leavenotify(xcb_leave_notify_event_t *ev)
 
     globalconf.timestamp = ev->time;
 
+    /*
+     * Ignore events with non-normal modes. Those are because a grab
+     * activated/deactivated. Everything will be "back to normal" after the
+     * grab.
+     */
     if(ev->mode != XCB_NOTIFY_MODE_NORMAL)
         return;
 
-    /* Ignore leave with detail inferior (we were left for a window contained in
-     * our window, so technically the pointer is still inside of this window).
-     */
-    if(ev->detail != XCB_NOTIFY_DETAIL_INFERIOR && (c = client_getbyframewin(ev->event)))
+    if((c = client_getbyframewin(ev->event)))
     {
-        luaA_object_push(L, c);
-        luaA_object_emit_signal(L, -1, "mouse::leave", 0);
+        /* The window was left in some way, so definitely no titlebar has the
+         * mouse cursor.
+         */
+        lua_pushnil(L);
+        event_drawable_under_mouse(L, -1);
+        lua_pop(L, 1);
+
+        /* If detail is inferior, it means that the cursor is now in some child
+         * window of our window. Thus, the titlebar was left, but now the cursor
+         * is in the actual child window. Thus, ignore detail=Inferior for
+         * leaving client windows.
+         */
+        if(ev->detail != XCB_NOTIFY_DETAIL_INFERIOR) {
+            luaA_object_push(L, c);
+            luaA_object_emit_signal(L, -1, "mouse::leave", 0);
+            lua_pop(L, 1);
+        }
+    } else if(ev->detail != XCB_NOTIFY_DETAIL_INFERIOR) {
+        /* Some window was left. This must be a drawin. Ignore detail=Inferior,
+         * because this means that some child window now contains the mouse
+         * cursor, i.e. a systray window. Everything else is a real 'leave'.
+         */
+        lua_pushnil(L);
+        event_drawable_under_mouse(L, -1);
         lua_pop(L, 1);
     }
-
-    lua_pushnil(L);
-    event_drawable_under_mouse(L, -1);
-    lua_pop(L, 1);
 }
 
 /** The enter notify event handler.
@@ -623,10 +632,24 @@ event_handle_enternotify(xcb_enter_notify_event_t *ev)
 
     globalconf.timestamp = ev->time;
 
+    /*
+     * Ignore events with non-normal modes. Those are because a grab
+     * activated/deactivated. Everything will be "back to normal" after the
+     * grab.
+     */
     if(ev->mode != XCB_NOTIFY_MODE_NORMAL)
         return;
 
-    if((drawin = drawin_getbywin(ev->event)))
+    /*
+     * We ignore events with detail "inferior".  This detail means that the
+     * cursor was previously inside of a child window and now left that child
+     * window. For our purposes, the cursor was already inside our window
+     * before.
+     * One exception are titlebars: They are not their own window, but are
+     * "outside of the actual client window".
+     */
+
+    if(ev->detail != XCB_NOTIFY_DETAIL_INFERIOR && (drawin = drawin_getbywin(ev->event)))
     {
         luaA_object_push(L, drawin);
         luaA_object_push_item(L, -1, drawin->drawable);
@@ -637,22 +660,25 @@ event_handle_enternotify(xcb_enter_notify_event_t *ev)
     if((c = client_getbyframewin(ev->event)))
     {
         luaA_object_push(L, c);
-        /* Ignore enter with detail inferior: The pointer was previously inside
-         * of a child window, so technically this isn't a 'real' enter.
+        /* Detail=Inferior means that a child of the frame window now contains
+         * the mouse cursor, i.e. the actual client now has the cursor. All
+         * other details mean that the client itself was really left.
          */
-        if (ev->detail != XCB_NOTIFY_DETAIL_INFERIOR)
+        if(ev->detail != XCB_NOTIFY_DETAIL_INFERIOR) {
             luaA_object_emit_signal(L, -1, "mouse::enter", 0);
+        }
 
         drawable_t *d = client_get_drawable(c, ev->event_x, ev->event_y);
         if (d)
         {
             luaA_object_push_item(L, -1, d);
-            event_drawable_under_mouse(L, -1);
-            lua_pop(L, 1);
+        } else {
+            lua_pushnil(L);
         }
-        lua_pop(L, 1);
+        event_drawable_under_mouse(L, -1);
+        lua_pop(L, 2);
     }
-    else if (ev->event == globalconf.screen->root) {
+    else if (ev->detail != XCB_NOTIFY_DETAIL_INFERIOR && ev->event == globalconf.screen->root) {
         /* When there are multiple X screens with awesome running separate
          * instances, reset focus.
          */
@@ -864,7 +890,7 @@ event_handle_randr_output_change_notify(xcb_randr_notify_event_t *ev)
         xcb_randr_output_t output = ev->u.oc.output;
         uint8_t connection = ev->u.oc.connection;
         const char *connection_str = NULL;
-        xcb_randr_get_output_info_reply_t *info = NULL;
+        xcb_randr_get_output_info_reply_t *info;
         lua_State *L = globalconf_get_lua_State();
 
         /* The following explicitly uses XCB_CURRENT_TIME since we want to know
