@@ -1,7 +1,7 @@
 /*
  * event.h - event handlers header
  *
- * Copyright © 2007-2008 Julien Danjou <julien@danjou.info>
+ * Copyright © 2007-2009 Julien Danjou <julien@danjou.info>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,21 +22,46 @@
 #ifndef AWESOME_EVENT_H
 #define AWESOME_EVENT_H
 
-#include "systray.h"
-#include "wibox.h"
-#include "client.h"
+#include "banning.h"
+#include "globalconf.h"
+#include "stack.h"
+
+#include <xcb/xcb.h>
+
+/* luaa.c */
+void luaA_emit_refresh(void);
+
+/* objects/drawin.c */
+void drawin_refresh(void);
+
+/* objects/client.c */
+void client_refresh(void);
+void client_focus_refresh(void);
+void client_destroy_later(void);
+
+/* objects/screen.c */
+void screen_refresh(void);
+
+/* xkb.c */
+void xkb_refresh(void);
 
 static inline int
 awesome_refresh(void)
 {
+    xkb_refresh();
+    screen_refresh();
+    luaA_emit_refresh();
+    drawin_refresh();
+    client_refresh();
     banning_refresh();
-    wibox_refresh();
-    systray_refresh();
-    client_stack_refresh();
+    stack_refresh();
+    client_destroy_later();
     return xcb_flush(globalconf.connection);
 }
 
-void event_handle(xcb_generic_event_t *event);
+void event_init(void);
+void event_handle(xcb_generic_event_t *);
+void event_drawable_under_mouse(lua_State *, int);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
