@@ -19,27 +19,24 @@
  *
  */
 
+#include "ewmh.h"
+#include "objects/client.h"
+#include "objects/tag.h"
+#include "common/atoms.h"
+#include "xwindow.h"
+
 #include <sys/types.h>
 #include <unistd.h>
 
 #include <xcb/xcb.h>
 #include <xcb/xcb_atom.h>
 
-#include "ewmh.h"
-#include "objects/tag.h"
-#include "screen.h"
-#include "objects/client.h"
-#include "luaa.h"
-#include "common/atoms.h"
-#include "common/buffer.h"
-#include "common/xutil.h"
-
 #define _NET_WM_STATE_REMOVE 0
 #define _NET_WM_STATE_ADD 1
 #define _NET_WM_STATE_TOGGLE 2
 
 /** Update client EWMH hints.
- * \param c The client.
+ * \param L The Lua VM state.
  */
 static int
 ewmh_client_update_hints(lua_State *L)
@@ -52,9 +49,9 @@ ewmh_client_update_hints(lua_State *L)
         state[i++] = _NET_WM_STATE_MODAL;
     if(c->fullscreen)
         state[i++] = _NET_WM_STATE_FULLSCREEN;
-    if(c->maximized_vertical)
+    if(c->maximized_vertical || c->maximized)
         state[i++] = _NET_WM_STATE_MAXIMIZED_VERT;
-    if(c->maximized_horizontal)
+    if(c->maximized_horizontal || c->maximized)
         state[i++] = _NET_WM_STATE_MAXIMIZED_HORZ;
     if(c->sticky)
         state[i++] = _NET_WM_STATE_STICKY;
@@ -75,19 +72,6 @@ ewmh_client_update_hints(lua_State *L)
     return 0;
 }
 
-/** Update the desktop geometry.
- */
-static void
-ewmh_update_desktop_geometry(void)
-{
-    area_t geom = screen_area_get(&globalconf.screens.tab[0], false);
-    uint32_t sizes[] = { geom.width, geom.height };
-
-    xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-                        globalconf.screen->root,
-                        _NET_DESKTOP_GEOMETRY, XCB_ATOM_CARDINAL, 32, countof(sizes), sizes);
-}
-
 static int
 ewmh_update_net_active_window(lua_State *L)
 {
@@ -121,6 +105,23 @@ ewmh_update_net_client_list(lua_State *L)
     return 0;
 }
 
+static int
+ewmh_client_update_frame_extents(lua_State *L)
+{
+    client_t *c = luaA_checkudata(L, 1, &client_class);;
+    uint32_t extents[4];
+
+    extents[0] = c->border_width + c->titlebar[CLIENT_TITLEBAR_LEFT].size;
+    extents[1] = c->border_width + c->titlebar[CLIENT_TITLEBAR_RIGHT].size;
+    extents[2] = c->border_width + c->titlebar[CLIENT_TITLEBAR_TOP].size;
+    extents[3] = c->border_width + c->titlebar[CLIENT_TITLEBAR_BOTTOM].size;
+
+    xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
+            c->window, _NET_FRAME_EXTENTS, XCB_ATOM_CARDINAL, 32, 4, extents);
+
+    return 0;
+}
+
 void
 ewmh_init(void)
 {
@@ -137,8 +138,8 @@ ewmh_init(void)
         _NET_CURRENT_DESKTOP,
         _NET_DESKTOP_NAMES,
         _NET_ACTIVE_WINDOW,
-        _NET_DESKTOP_GEOMETRY,
         _NET_CLOSE_WINDOW,
+        _NET_FRAME_EXTENTS,
         _NET_WM_NAME,
         _NET_WM_STRUT_PARTIAL,
         _NET_WM_ICON_NAME,
@@ -197,27 +198,69 @@ ewmh_init(void)
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
                         father, _NET_WM_NAME, UTF8_STRING, 8, 7, "awesome");
 
+    /* Set an instance, just because we can */
+    xwindow_set_class_instance(father);
+
     /* set the window manager PID */
     i = getpid();
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
                         father, _NET_WM_PID, XCB_ATOM_CARDINAL, 32, 1, &i);
+}
 
-    ewmh_update_desktop_geometry();
+static void
+ewmh_update_maximize(bool h, bool status, bool toggle)
+{
+    lua_State *L = globalconf_get_lua_State();
 
-    luaA_class_connect_signal(globalconf.L, &client_class, "focus", ewmh_update_net_active_window);
-    luaA_class_connect_signal(globalconf.L, &client_class, "unfocus", ewmh_update_net_active_window);
-    luaA_class_connect_signal(globalconf.L, &client_class, "manage", ewmh_update_net_client_list);
-    luaA_class_connect_signal(globalconf.L, &client_class, "unmanage", ewmh_update_net_client_list);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::modal" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::fullscreen" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::maximized_horizontal" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::maximized_vertical" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::sticky" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::skip_taskbar" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::above" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::below" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::minimized" , ewmh_client_update_hints);
-    luaA_class_connect_signal(globalconf.L, &client_class, "property::urgent" , ewmh_client_update_hints);
+    if (h)
+        lua_pushstring(L, "client_maximize_horizontal");
+    else
+        lua_pushstring(L, "client_maximize_vertical");
+
+    /* Create table argument with raise=true. */
+    lua_newtable(L);
+    lua_pushstring(L, "toggle");
+    lua_pushboolean(L, toggle);
+    lua_settable(L, -3);
+    lua_pushstring(L, "status");
+    lua_pushboolean(L, status);
+    lua_settable(L, -3);
+
+    luaA_object_emit_signal(L, -3, "request::geometry", 2);
+}
+
+void
+ewmh_init_lua(void)
+{
+    lua_State *L = globalconf_get_lua_State();
+
+    luaA_class_connect_signal(L, &client_class, "focus", ewmh_update_net_active_window);
+    luaA_class_connect_signal(L, &client_class, "unfocus", ewmh_update_net_active_window);
+    luaA_class_connect_signal(L, &client_class, "manage", ewmh_update_net_client_list);
+    luaA_class_connect_signal(L, &client_class, "unmanage", ewmh_update_net_client_list);
+    luaA_class_connect_signal(L, &client_class, "property::modal" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::fullscreen" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::maximized_horizontal" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::maximized_vertical" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::maximized" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::sticky" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::skip_taskbar" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::above" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::below" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::minimized" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::urgent" , ewmh_client_update_hints);
+    luaA_class_connect_signal(L, &client_class, "property::titlebar_top" , ewmh_client_update_frame_extents);
+    luaA_class_connect_signal(L, &client_class, "property::titlebar_bottom" , ewmh_client_update_frame_extents);
+    luaA_class_connect_signal(L, &client_class, "property::titlebar_right" , ewmh_client_update_frame_extents);
+    luaA_class_connect_signal(L, &client_class, "property::titlebar_left" , ewmh_client_update_frame_extents);
+    luaA_class_connect_signal(L, &client_class, "property::border_width" , ewmh_client_update_frame_extents);
+    luaA_class_connect_signal(L, &client_class, "manage", ewmh_client_update_frame_extents);
+    /* NET_CURRENT_DESKTOP handling */
+    luaA_class_connect_signal(L, &client_class, "focus", ewmh_update_net_current_desktop);
+    luaA_class_connect_signal(L, &client_class, "unfocus", ewmh_update_net_current_desktop);
+    luaA_class_connect_signal(L, &client_class, "tagged", ewmh_update_net_current_desktop);
+    luaA_class_connect_signal(L, &client_class, "untagged", ewmh_update_net_current_desktop);
+    luaA_class_connect_signal(L, &tag_class, "property::selected", ewmh_update_net_current_desktop);
 }
 
 /** Set the client list in stacking order, bottom to top.
@@ -246,14 +289,15 @@ ewmh_update_net_numbers_of_desktop(void)
 			_NET_NUMBER_OF_DESKTOPS, XCB_ATOM_CARDINAL, 32, 1, &count);
 }
 
-void
-ewmh_update_net_current_desktop(void)
+int
+ewmh_update_net_current_desktop(lua_State *L)
 {
-    uint32_t idx = tags_get_first_selected_index();
+    uint32_t idx = tags_get_current_or_first_selected_index();
 
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
                         globalconf.screen->root,
                         _NET_CURRENT_DESKTOP, XCB_ATOM_CARDINAL, 32, 1, &idx);
+    return 0;
 }
 
 void
@@ -278,100 +322,130 @@ ewmh_update_net_desktop_names(void)
 static void
 ewmh_process_state_atom(client_t *c, xcb_atom_t state, int set)
 {
-    luaA_object_push(globalconf.L, c);
+    lua_State *L = globalconf_get_lua_State();
+    luaA_object_push(L, c);
 
     if(state == _NET_WM_STATE_STICKY)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_sticky(globalconf.L, -1, false);
+            client_set_sticky(L, -1, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_sticky(globalconf.L, -1, true);
+            client_set_sticky(L, -1, true);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_sticky(globalconf.L, -1, !c->sticky);
+            client_set_sticky(L, -1, !c->sticky);
     }
     else if(state == _NET_WM_STATE_SKIP_TASKBAR)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_skip_taskbar(globalconf.L, -1, false);
+            client_set_skip_taskbar(L, -1, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_skip_taskbar(globalconf.L, -1, true);
+            client_set_skip_taskbar(L, -1, true);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_skip_taskbar(globalconf.L, -1, !c->skip_taskbar);
+            client_set_skip_taskbar(L, -1, !c->skip_taskbar);
     }
     else if(state == _NET_WM_STATE_FULLSCREEN)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_fullscreen(globalconf.L, -1, false);
+            client_set_fullscreen(L, -1, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_fullscreen(globalconf.L, -1, true);
+            client_set_fullscreen(L, -1, true);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_fullscreen(globalconf.L, -1, !c->fullscreen);
+            client_set_fullscreen(L, -1, !c->fullscreen);
     }
     else if(state == _NET_WM_STATE_MAXIMIZED_HORZ)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_maximized_horizontal(globalconf.L, -1, false);
+            ewmh_update_maximize(true, false, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_maximized_horizontal(globalconf.L, -1, true);
+            ewmh_update_maximize(true, true, false);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_maximized_horizontal(globalconf.L, -1, !c->maximized_horizontal);
+            ewmh_update_maximize(true, false, true);
     }
     else if(state == _NET_WM_STATE_MAXIMIZED_VERT)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_maximized_vertical(globalconf.L, -1, false);
+            ewmh_update_maximize(false, false, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_maximized_vertical(globalconf.L, -1, true);
+            ewmh_update_maximize(false, true, false);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_maximized_vertical(globalconf.L, -1, !c->maximized_vertical);
+            ewmh_update_maximize(false, false, true);
     }
     else if(state == _NET_WM_STATE_ABOVE)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_above(globalconf.L, -1, false);
+            client_set_above(L, -1, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_above(globalconf.L, -1, true);
+            client_set_above(L, -1, true);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_above(globalconf.L, -1, !c->above);
+            client_set_above(L, -1, !c->above);
     }
     else if(state == _NET_WM_STATE_BELOW)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_below(globalconf.L, -1, false);
+            client_set_below(L, -1, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_below(globalconf.L, -1, true);
+            client_set_below(L, -1, true);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_below(globalconf.L, -1, !c->below);
+            client_set_below(L, -1, !c->below);
     }
     else if(state == _NET_WM_STATE_MODAL)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_modal(globalconf.L, -1, false);
+            client_set_modal(L, -1, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_modal(globalconf.L, -1, true);
+            client_set_modal(L, -1, true);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_modal(globalconf.L, -1, !c->modal);
+            client_set_modal(L, -1, !c->modal);
     }
     else if(state == _NET_WM_STATE_HIDDEN)
     {
         if(set == _NET_WM_STATE_REMOVE)
-            client_set_minimized(globalconf.L, -1, false);
+            client_set_minimized(L, -1, false);
         else if(set == _NET_WM_STATE_ADD)
-            client_set_minimized(globalconf.L, -1, true);
+            client_set_minimized(L, -1, true);
         else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_minimized(globalconf.L, -1, !c->minimized);
+            client_set_minimized(L, -1, !c->minimized);
     }
     else if(state == _NET_WM_STATE_DEMANDS_ATTENTION)
     {
-        if(set == _NET_WM_STATE_REMOVE)
-            client_set_urgent(globalconf.L, -1, false);
-        else if(set == _NET_WM_STATE_ADD)
-            client_set_urgent(globalconf.L, -1, true);
-        else if(set == _NET_WM_STATE_TOGGLE)
-            client_set_urgent(globalconf.L, -1, !c->urgent);
+        if(set == _NET_WM_STATE_REMOVE) {
+            lua_pushboolean(L, false);
+            luaA_object_emit_signal(L, -2, "request::urgent", 1);
+        }
+        else if(set == _NET_WM_STATE_ADD) {
+            lua_pushboolean(L, true);
+            luaA_object_emit_signal(L, -2, "request::urgent", 1);
+        }
+        else if(set == _NET_WM_STATE_TOGGLE) {
+            lua_pushboolean(L, !c->urgent);
+            luaA_object_emit_signal(L, -2, "request::urgent", 1);
+        }
     }
 
-    lua_pop(globalconf.L, 1);
+    lua_pop(L, 1);
+}
+
+static void
+ewmh_process_desktop(client_t *c, uint32_t desktop)
+{
+    lua_State *L = globalconf_get_lua_State();
+    int idx = desktop;
+    if(desktop == 0xffffffff)
+    {
+        luaA_object_push(L, c);
+        lua_pushboolean(L, true);
+        luaA_object_emit_signal(L, -2, "request::tag", 1);
+        /* Pop the client, arguments are already popped */
+        lua_pop(L, 1);
+    }
+    else if (idx >= 0 && idx < globalconf.tags.len)
+    {
+        luaA_object_push(L, c);
+        luaA_object_push(L, globalconf.tags.tab[idx]);
+        luaA_object_emit_signal(L, -2, "request::tag", 1);
+        /* Pop the client, arguments are already popped */
+        lua_pop(L, 1);
+    }
 }
 
 int
@@ -380,7 +454,16 @@ ewmh_process_client_message(xcb_client_message_event_t *ev)
     client_t *c;
 
     if(ev->type == _NET_CURRENT_DESKTOP)
-        tag_view_only_byindex(ev->data.data32[0]);
+    {
+        int idx = ev->data.data32[0];
+        if (idx >= 0 && idx < globalconf.tags.len)
+        {
+            lua_State *L = globalconf_get_lua_State();
+            luaA_object_push(L, globalconf.tags.tab[idx]);
+            luaA_object_emit_signal(L, -1, "request::select", 0);
+            lua_pop(L, 1);
+        }
+    }
     else if(ev->type == _NET_CLOSE_WINDOW)
     {
         if((c = client_getbywin(ev->window)))
@@ -390,17 +473,7 @@ ewmh_process_client_message(xcb_client_message_event_t *ev)
     {
         if((c = client_getbywin(ev->window)))
         {
-            if(ev->data.data32[0] == 0xffffffff)
-                c->sticky = true;
-            else
-                for(int i = 0; i < globalconf.tags.len; i++)
-                    if((int)ev->data.data32[0] == i)
-                    {
-                        luaA_object_push(globalconf.L, globalconf.tags.tab[i]);
-                        tag_client(c);
-                    }
-                    else
-                        untag_client(c, globalconf.tags.tab[i]);
+            ewmh_process_desktop(c, ev->data.data32[0]);
         }
     }
     else if(ev->type == _NET_WM_STATE)
@@ -416,8 +489,18 @@ ewmh_process_client_message(xcb_client_message_event_t *ev)
     else if(ev->type == _NET_ACTIVE_WINDOW)
     {
         if((c = client_getbywin(ev->window))) {
-            client_focus(c);
-            client_raise(c);
+            lua_State *L = globalconf_get_lua_State();
+            luaA_object_push(L, c);
+            lua_pushstring(L, "ewmh");
+
+            /* Create table argument with raise=true. */
+            lua_newtable(L);
+            lua_pushstring(L, "raise");
+            lua_pushboolean(L, true);
+            lua_settable(L, -3);
+
+            luaA_object_emit_signal(L, -3, "request::activate", 2);
+            lua_pop(L, 1);
         }
     }
 
@@ -491,9 +574,10 @@ ewmh_client_check_hints(client_t *c)
 {
     xcb_atom_t *state;
     void *data = NULL;
-    int desktop;
     xcb_get_property_cookie_t c0, c1, c2;
     xcb_get_property_reply_t *reply;
+    bool is_h_max = false;
+    bool is_v_max = false;
 
     /* Send the GetProperty requests which will be processed later */
     c0 = xcb_get_property_unchecked(globalconf.connection, false, c->window,
@@ -508,25 +592,7 @@ ewmh_client_check_hints(client_t *c)
     reply = xcb_get_property_reply(globalconf.connection, c0, NULL);
     if(reply && reply->value_len && (data = xcb_get_property_value(reply)))
     {
-        desktop = *(uint32_t *) data;
-        if(desktop == -1)
-            c->sticky = true;
-        else if (desktop >= 0 && desktop < globalconf.tags.len)
-            for(int i = 0; i < globalconf.tags.len; i++)
-                if(desktop == i)
-                {
-                    luaA_object_push(globalconf.L, globalconf.tags.tab[i]);
-                    tag_client(c);
-                }
-                else
-                    untag_client(c, globalconf.tags.tab[i]);
-        else
-            /* Value out of bounds, just give it the first tag */
-            if (globalconf.tags.len > 0)
-            {
-                luaA_object_push(globalconf.L, globalconf.tags.tab[0]);
-                tag_client(c);
-            }
+        ewmh_process_desktop(c, *(uint32_t *) data);
     }
 
     p_delete(&reply);
@@ -536,7 +602,34 @@ ewmh_client_check_hints(client_t *c)
     {
         state = (xcb_atom_t *) data;
         for(int i = 0; i < xcb_get_property_value_length(reply) / ssizeof(xcb_atom_t); i++)
-            ewmh_process_state_atom(c, state[i], _NET_WM_STATE_ADD);
+            if (state[i] == _NET_WM_STATE_MAXIMIZED_HORZ)
+                is_h_max = true;
+            else if (state[i] == _NET_WM_STATE_MAXIMIZED_VERT)
+                is_v_max = true;
+            else
+                ewmh_process_state_atom(c, state[i], _NET_WM_STATE_ADD);
+    }
+
+    /* Check maximization manually */
+    if (is_h_max && is_v_max) {
+        lua_State *L = globalconf_get_lua_State();
+        luaA_object_push(L, c);
+        client_set_maximized(L, -1, true);
+        lua_pop(L, 1);
+    }
+    else if(is_h_max)
+    {
+        lua_State *L = globalconf_get_lua_State();
+        luaA_object_push(L, c);
+        client_set_maximized_horizontal(L, -1, true);
+        lua_pop(L, 1);
+    }
+    else if(is_v_max)
+    {
+        lua_State *L = globalconf_get_lua_State();
+        luaA_object_push(L, c);
+        client_set_maximized_vertical(L, -1, true);
+        lua_pop(L, 1);
     }
 
     p_delete(&reply);
@@ -544,6 +637,7 @@ ewmh_client_check_hints(client_t *c)
     reply = xcb_get_property_reply(globalconf.connection, c2, NULL);
     if(reply && (data = xcb_get_property_value(reply)))
     {
+        c->has_NET_WM_WINDOW_TYPE = true;
         state = (xcb_atom_t *) data;
         for(int i = 0; i < xcb_get_property_value_length(reply) / ssizeof(xcb_atom_t); i++)
             if(state[i] == _NET_WM_WINDOW_TYPE_DESKTOP)
@@ -560,14 +654,14 @@ ewmh_client_check_hints(client_t *c)
                 c->type = MAX(c->type, WINDOW_TYPE_TOOLBAR);
             else if(state[i] == _NET_WM_WINDOW_TYPE_UTILITY)
                 c->type = MAX(c->type, WINDOW_TYPE_UTILITY);
-    }
+    } else
+        c->has_NET_WM_WINDOW_TYPE = false;
 
     p_delete(&reply);
 }
 
 /** Process the WM strut of a client.
  * \param c The client.
- * \param strut_r (Optional) An existing reply.
  */
 void
 ewmh_process_client_strut(client_t *c)
@@ -611,9 +705,10 @@ ewmh_process_client_strut(client_t *c)
             c->strut.bottom_start_x = strut[10];
             c->strut.bottom_end_x = strut[11];
 
-            luaA_object_push(globalconf.L, c);
-            luaA_object_emit_signal(globalconf.L, -1, "property::struts", 0);
-            lua_pop(globalconf.L, 1);
+            lua_State *L = globalconf_get_lua_State();
+            luaA_object_push(L, c);
+            luaA_object_emit_signal(L, -1, "property::struts", 0);
+            lua_pop(L, 1);
         }
     }
 
@@ -632,40 +727,62 @@ ewmh_window_icon_get_unchecked(xcb_window_t w)
 }
 
 static cairo_surface_t *
+ewmh_window_icon_from_reply_next(uint32_t **data, uint32_t *data_end)
+{
+    uint32_t width, height;
+    uint64_t data_len;
+    uint32_t *icon_data;
+
+    if(data_end - *data <= 2)
+        return NULL;
+
+    width = (*data)[0];
+    height = (*data)[1];
+
+    /* Check that we have enough data, handling overflow */
+    data_len = width * (uint64_t) height;
+    if (width < 1 || height < 1 || data_len > (uint64_t) (data_end - *data) - 2)
+        return NULL;
+
+    icon_data = *data + 2;
+    *data += 2 + data_len;
+    return draw_surface_from_data(width, height, icon_data);
+}
+
+static cairo_surface_array_t
 ewmh_window_icon_from_reply(xcb_get_property_reply_t *r)
 {
-    uint32_t *data;
-    uint64_t len;
+    uint32_t *data, *data_end;
+    cairo_surface_array_t result;
+    cairo_surface_t *s;
 
-    if(!r || r->type != XCB_ATOM_CARDINAL || r->format != 32 || r->length < 2)
-        return 0;
+    cairo_surface_array_init(&result);
+    if(!r || r->type != XCB_ATOM_CARDINAL || r->format != 32)
+        return result;
 
-    data = (uint32_t *) xcb_get_property_value(r);
-    if (!data)
-        return 0;
+    data = (uint32_t*) xcb_get_property_value(r);
+    data_end = &data[r->length];
+    if(!data)
+        return result;
 
-    /* Check that the property is as long as it should be, handling integer
-     * overflow. <uint32_t> times <another uint32_t casted to uint64_t> always
-     * fits into an uint64_t and thus this multiplication cannot overflow.
-     */
-    len = data[0] * (uint64_t) data[1];
-    if (!data[0] || !data[1] || len > r->length - 2)
-        return 0;
+    while ((s = ewmh_window_icon_from_reply_next(&data, data_end)) != NULL) {
+        cairo_surface_array_push(&result, s);
+    }
 
-    return draw_surface_from_data(data[0], data[1], data + 2);
+    return result;
 }
 
 /** Get NET_WM_ICON.
  * \param cookie The cookie.
- * \return The number of elements on stack.
+ * \return An array of icons.
  */
-cairo_surface_t *
+cairo_surface_array_t
 ewmh_window_icon_get_reply(xcb_get_property_cookie_t cookie)
 {
     xcb_get_property_reply_t *r = xcb_get_property_reply(globalconf.connection, cookie, NULL);
-    cairo_surface_t *surface = ewmh_window_icon_from_reply(r);
+    cairo_surface_array_t result = ewmh_window_icon_from_reply(r);
     p_delete(&r);
-    return surface;
+    return result;
 }
 
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
