@@ -156,7 +156,6 @@ drawin_systray_kickout(drawin_t *w)
         /* Who! Check that we're not deleting a drawin with a systray, because it
          * may be its parent. If so, we reparent to root before, otherwise it will
          * hurt very much. */
-        systray_cleanup();
         xcb_reparent_window(globalconf.connection,
                             globalconf.systray.window,
                             globalconf.screen->root,
@@ -238,6 +237,25 @@ drawin_refresh(void)
     }
 }
 
+/** Get all drawins into a table.
+ * @treturn table A table with drawins.
+ * @function get
+ */
+static int
+luaA_drawin_get(lua_State *L)
+{
+    int i = 1;
+
+    lua_newtable(L);
+
+    foreach(d, globalconf.drawins) {
+        luaA_object_push(L, *d);
+        lua_rawseti(L, -2, i++);
+    }
+
+    return 1;
+}
+
 /** Move and/or resize a drawin
  * \param L The Lua VM state.
  * \param udx The index of the drawin.
@@ -746,6 +764,7 @@ drawin_class_setup(lua_State *L)
     static const struct luaL_Reg drawin_methods[] =
     {
         LUA_CLASS_METHODS(drawin)
+        { "get", luaA_drawin_get },
         { "__call", luaA_drawin_new },
         { NULL, NULL }
     };
