@@ -57,6 +57,11 @@
 #include <xcb/xinerama.h>
 #include <xcb/randr.h>
 
+/* The XID that is used on fake screens. X11 guarantees that the top three bits
+ * of a valid XID are zero, so this will not clash with anything.
+ */
+#define FAKE_SCREEN_XID ((uint32_t) 0xffffffff)
+
 /** Screen is a table where indexes are screen numbers. You can use `screen[1]`
  * to get access to the first screen, etc. Alternatively, if RANDR information
  * is available, you can use output names for finding screen objects.
@@ -66,12 +71,12 @@
  */
 
 /**
- * @signal .primary_changed
+ * @signal primary_changed
  */
 
 /**
  * This signal is emitted when a new screen is added to the current setup.
- * @signal .added
+ * @signal added
  */
 
 /**
@@ -80,13 +85,13 @@
  */
 
 /** This signal is emitted when the list of available screens changes.
- * @signal .list
+ * @signal list
  */
 
 /** When 2 screens are swapped
  * @tparam screen screen The other screen
  * @tparam boolean is_source If self is the source or the destination of the swap
- * @signal .swapped
+ * @signal swapped
  */
 
  /**
@@ -112,14 +117,34 @@
  */
 
 /**
- * The screen number.
+ * The internal screen number.
  *
- * An integer greater than 1 and smaller than `screen.count()`. Please note that
- * the screen order isn't always mirroring the screen logical position.
+ * * The indeces are a continuous sequence from 1 to `screen.count()`.
+ * * It is **NOT** related to the actual screen position relative to each
+ *   other.
+ * * 1 is **NOT** necessarily the primary screen.
+ * * When screens are added and removed indices **CAN** change.
+ *
+ * If you really want to keep an array of screens you should use something
+ * along:
+ *
+ *     local myscreens = setmetatable({}. {__mode="k"})
+ *     myscreens[ screen[1] ] = "mydata"
+ *
+ * But it might be a better option to simply store the data directly in the
+ * screen object as:
+ *
+ *     screen[1].mydata = "mydata"
+ *
+ * Remember that screens are also objects, so if you only want to store a simple
+ * property, you can do it directly:
+ *
+ *     screen[1].answer = 42
  *
  * **Immutable:** true
  * @property index
  * @param integer
+ * @see screen
  */
 
 /**
@@ -224,8 +249,11 @@ luaA_checkscreen(lua_State *L, int sidx)
     {
         int screen = lua_tointeger(L, sidx);
         if(screen < 1 || screen > globalconf.screens.len)
-            luaL_error(L, "invalid screen number: %d (of %d existing)",
-                    screen, globalconf.screens.len);
+        {
+            luaA_warn(L, "invalid screen number: %d (of %d existing)", screen, globalconf.screens.len);
+            lua_pushnil(L);
+            return NULL;
+        }
         return globalconf.screens.tab[screen - 1];
     } else
         return luaA_checkudata(L, sidx, &screen_class);
@@ -270,6 +298,7 @@ screen_add(lua_State *L, screen_array_t *screens)
     screen_t *new_screen = screen_new(L);
     luaA_object_ref(L, -1);
     screen_array_append(screens, new_screen);
+    new_screen->xid = XCB_NONE;
     return new_screen;
 }
 
@@ -640,6 +669,7 @@ screen_refresh(void)
     globalconf.screen_need_refresh = false;
 
     screen_array_t new_screens;
+    screen_array_t removed_screens;
     lua_State *L = globalconf_get_lua_State();
     bool list_changed = false;
 
@@ -651,6 +681,10 @@ screen_refresh(void)
 
     screen_deduplicate(L, &new_screens);
 
+    /* Running without any screens at all is no fun. */
+    if (new_screens.len == 0)
+        screen_scan_x11(L, &new_screens);
+
     /* Add new screens */
     foreach(new_screen, new_screens) {
         bool found = false;
@@ -669,24 +703,28 @@ screen_refresh(void)
     }
 
     /* Remove screens which are gone */
+    screen_array_init(&removed_screens);
     for(int i = 0; i < globalconf.screens.len; i++) {
         screen_t *old_screen = globalconf.screens.tab[i];
-        bool found = false;
+        bool found = old_screen->xid == FAKE_SCREEN_XID;
         foreach(new_screen, new_screens)
             found |= (*new_screen)->xid == old_screen->xid;
         if(!found) {
             screen_array_take(&globalconf.screens, i);
             i--;
 
-            luaA_object_push(L, old_screen);
-            screen_removed(L, -1);
-            lua_pop(L, 1);
-            luaA_object_unref(L, old_screen);
-            old_screen->valid = false;
-
+            screen_array_append(&removed_screens, old_screen);
             list_changed = true;
         }
     }
+    foreach(old_screen, removed_screens) {
+        luaA_object_push(L, *old_screen);
+        screen_removed(L, -1);
+        lua_pop(L, 1);
+        (*old_screen)->valid = false;
+        luaA_object_unref(L, *old_screen);
+    }
+    screen_array_wipe(&removed_screens);
 
     /* Update changed screens */
     foreach(existing_screen, globalconf.screens)
@@ -1029,7 +1067,9 @@ luaA_screen_module_index(lua_State *L)
             foreach(output, (*screen)->outputs)
                 if(A_STREQ(output->name, name))
                     return luaA_object_push(L, *screen);
-        luaL_error(L, "Unknown screen output name: %s", name);
+        luaA_warn(L, "Unknown screen output name: %s", name);
+        lua_pushnil(L);
+        return 1;
     }
 
     return luaA_object_push(L, luaA_checkscreen(L, 2));
@@ -1139,6 +1179,7 @@ luaA_screen_fake_add(lua_State *L)
     s->geometry.y = y;
     s->geometry.width = width;
     s->geometry.height = height;
+    s->xid = FAKE_SCREEN_XID;
 
     screen_added(L, s);
     luaA_class_emit_signal(L, &screen_class, "list", 0);
@@ -1159,6 +1200,11 @@ luaA_screen_fake_remove(lua_State *L)
         /* WTF? */
         return 0;
 
+    if (globalconf.screens.len == 1) {
+        luaA_warn(L, "Removing last screen through fake_remove(). "
+                "This is a very, very, very bad idea!");
+    }
+
     screen_array_take(&globalconf.screens, idx);
     luaA_object_push(L, s);
     screen_removed(L, -1);
@@ -1222,6 +1268,9 @@ luaA_screen_swap(lua_State *L)
             if(ref_s && ref_swap)
                 break;
         }
+        if(!ref_s || !ref_swap)
+            return luaL_error(L, "Invalid call to screen:swap()");
+
         /* swap ! */
         *ref_s = swap;
         *ref_swap = s;
@@ -1289,4 +1338,6 @@ screen_class_setup(lua_State *L)
                             NULL);
 }
 
+/* @DOC_cobject_COMMON@ */
+
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
