@@ -194,7 +194,7 @@ luaA_window_set_border_color(lua_State *L, window_t *window)
     const char *color_name = luaL_checklstring(L, -1, &len);
 
     if(color_name &&
-       color_init_reply(color_init_unchecked(&window->border_color, color_name, len)))
+       color_init_reply(color_init_unchecked(&window->border_color, color_name, len, globalconf.visual)))
     {
         window->border_need_update = true;
         luaA_object_emit_signal(L, -3, "property::border_color", 0);
@@ -356,7 +356,6 @@ window_set_xproperty(lua_State *L, xcb_window_t window, int prop_idx, int value_
 {
     xproperty_t *prop = luaA_find_xproperty(L, prop_idx);
     xcb_atom_t type;
-    uint8_t format;
     size_t len;
     uint32_t number;
     const void *data;
@@ -365,6 +364,7 @@ window_set_xproperty(lua_State *L, xcb_window_t window, int prop_idx, int value_
     {
         xcb_delete_property(globalconf.connection, window, prop->atom);
     } else {
+        uint8_t format;
         if(prop->type == PROP_STRING)
         {
             data = luaL_checklstring(L, value_idx, &len);
