@@ -27,10 +27,9 @@
 #include <lua.h>
 
 void spawn_init(void);
-void spawn_handle_reap(const char *);
-char * const * spawn_transform_commandline(char **);
 void spawn_start_notify(client_t *, const char *);
 int luaA_spawn(lua_State *);
+void spawn_child_exited(pid_t, int);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
