@@ -41,4 +41,11 @@ describe("gears.table", function()
             assert.is.equal(f(), nil)
         end)
     end)
+
+    describe("table.join", function()
+        it("nil argument", function()
+            local t = gtable.join({"a"}, nil, {"b"})
+            assert.is.same(t, {"a", "b"})
+        end)
+    end)
 end)
