@@ -7,6 +7,6 @@ require("lgi")
 _G.awesome = {version = "v9999"}
 
 -- "fix" some intentional beautiful breakage done by .travis.yml
-require("beautiful").init{}
+require("beautiful").init{a_key="a_value"}
 
 -- vim: filetype=lua:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
