@@ -22,11 +22,13 @@
 #ifndef AWESOME_STACK_H
 #define AWESOME_STACK_H
 
-#include "globalconf.h"
+typedef struct client_t client_t;
 
 void stack_client_remove(client_t *);
 void stack_client_push(client_t *);
 void stack_client_append(client_t *);
+void stack_windows(void);
+void stack_refresh(void);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
