@@ -1,3 +1,4 @@
+--DOC_GEN_IMAGE --DOC_HIDE
 local parent    = ... --DOC_NO_USAGE --DOC_HIDE
 local wibox     = require( "wibox"     ) --DOC_HIDE
 local awful     = { prompt = require("awful.prompt"),--DOC_HIDE
@@ -24,7 +25,7 @@ local naughty   = {} --DOC_HIDE
             end
         end,
         textbox              = atextbox,
-        history_path         = gfs.get_dir("cache") .. "/history",
+        history_path         = gfs.get_cache_dir() .. "/history",
     }
 
 parent:add( wibox.widget {    --DOC_HIDE
